#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.interface import Interface, invariant, Invalid
from zope.interface.interfaces import IInterface
from zope.app.container.constraints import containers, contains
from zope.app.container.interfaces import IContained, IOrderedContainer

from z3c.schema import email

class IStoreAttributes(IAnnotatable):
    """Marker interface that promises an implementing object
    can have ''IAttributes'' annotations.
    """

    
class IAttributes(Interface):
    """Store the attributes.
    """


class IAttributeContainer(IOrderedContainer):
    """Stores the attribute sheets.
    """
    contains('zw.vogon.attribute.interfaces.IAttributeSheet')


##############################################################################

class IBaseAttributeSchema(Interface):
    """Base schema of IAttributeFields
    """


class IAttributeSheet(IContained):
    """Contains the actual attributes.
    """
    containers('zw.vogon.attribute.interfaces.IAttributeContainer')

    __schema__ = schema.InterfaceField(
        title = _( u'label-attribute.IAttributes.__schema__',
                   u"Interface schema of attributes" ),
        description = _( u'help-attribute.IAttributes.__schema__',
                         u"" ),
        required = True,
        default = IBaseAttributeSchema )


    __title__ = schema.TextLine(
        title = _( u'label-attribute.IAttributeSheet.title',
                   u"Title" ),
        description = _( u'help-attribute.IAttributeSheet.title',
                         u"" ),
        required = True )

    __description__ = schema.Text(
        title = _( u'help-attribute-IAttributeSheet.description',
                   u"Description" ),
        description = _ ( u'help-attribute.IAttributeSheet.description',
                          u"" ),
        required = False )


#############################################################################
#############################################################################


class IStoreAttributeTemplates(IAnnotatable):
    """Marker interface that promises an implementing object
    can store ''IAttributeTemplates'' annotations.
    """


class IAttributeTemplates(Interface):
    """The folder containing the attribute category templates.
    """


class IAttributeCategoryContainer(IOrderedContainer):
    """Stores categories of attribute templates.
    """
    contains('zw.vogon.attribute.interfaces.IAttributeCategory')


##############################################################################


class IAttributeCategory(IContained):
    """Stores a category of templates. 
    """
    containers('zw.vogon.attribute.interfaces.IAttributeCategoryContainer')
    
    def getAttributeSchema():
        """Creates an attribute schema from the containing fields.
        """

    title = schema.TextLine(
        title = _( u"Title" ),
        description = _( u'help-attribute.IAttributeCategory.title',
                         u"" ),
        required = True )

    description = schema.Text(
        title = _( u"Description" ),
        description = _ ( u'help-attribute.IAttributeCategory.description',
                          u"" ),
        required = False )

    public = schema.Bool(
        title = _( u"Public" ),
        description = _( u"" ),
        required = True,
        default = True )

        


class IAttributeTemplateContainer(IOrderedContainer):
    """Stores attribute fields.
    """
    contains('zw.vogon.attribute.interfaces.IAttributeTemplate')


##############################################################################

class IAttributeType(IInterface):
    """This interface represents an attribute type.

    If an interface provides this interface type, then all objects
    providing this interface are considered attributes.
    """


class ReadonlyValueMissingError(Invalid):
    """A readonly field requires a default value for initialisation."""
    

class IAttributeTemplate(IContained):
    """An abstract attribute field.
    """
    containers('zw.vogon.attribute.interfaces.IAttributeTemplateContainer')

    title = schema.interfaces.IField['title']
    description = schema.interfaces.IField['description']
    required = schema.interfaces.IField['required']
    default = schema.interfaces.IField['default']

    public = schema.Bool(
        title = _( u"Public" ),
        description = _( u"" ),
        required = True,
        default = True )

    readonly = schema.Bool(
        title = _( u"Readonly" ),
        description = _( u"" ),
        required = True,
        default = False )


    @invariant
    def readonlyRequiresDefaultValue(ob):
        if ob.readonly and not ob.default:
            raise ReadonlyValueMissingError(ob)


class IBoolTemplate(IAttributeTemplate):
    """Template for a bool attribute.
    """

    default = schema.Bool(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                       field value"""),
        required = False,
        )

class ITextTemplate(IAttributeTemplate):
    """Template for a text attribute.
    """
    
    default = schema.Text(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                        field value"""),
        required = False,
        )
        
    cols = schema.Int(
        title = _( u"Columns" ),
        description = _( u"" ),
        required = True,
        default = 60 )

    rows = schema.Int(
        title = _( u"Rows" ),
        description = _( u"" ),
        required = True,
        default = 20 )


class IIntTemplate(IAttributeTemplate):
    """Template for an int attribute.
    """

    default = schema.Int(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                        field value"""),
        required = False,
        )

    min = schema.interfaces.IInt['min']
    max = schema.interfaces.IInt['max']

class IDatetimeTemplate(IAttributeTemplate):
    """Template for date and time attributes.
    """

    default = schema.Datetime(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                        field value"""),
        required = False,
        )

class IURITemplate(IAttributeTemplate):
    """Template for a URI attribute.
    """

    default = schema.URI(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                        field value"""),
        required = False,
        )

    cols = schema.Int(
        title = _( u"Columns" ),
        description = _( u"" ),
        required = True,
        default = 60 )
    

class IRFC822MailAddressTemplate(IAttributeTemplate):
    """Template for a RFC822 mail address attribute.
    """

    default = email.RFC822MailAddress(
        title=_(u"Default Value"),
        description=_(u"""The field default value may be None or a legal
                        field value"""),
        required = False,
        )

    cols = schema.Int(
        title = _( u"Columns" ),
        description = _( u"" ),
        required = True,
        default = 60 )


class IObjectTemplate(IAttributeTemplate):
    """Template for an object attribute.
    """




###########################################################################


class IAttributeField(schema.interfaces.IField):
    """An abstract field for attribute schemas.
    """
    
    public = schema.Bool(
        title = _( u"Public" ),
        required = True,
        default = True )

    readonly = schema.Bool(
        title = _( u"Readonly" ),
        description = _( u"A fix attribute is not showed on templating." ),
        required = True,
        default = False )

    @invariant
    def readonlyRequiresValue(ob):
        if ob.readonly and not ( ob.default or ob.missing_value ):
            raise ReadonlyValueMissingError(ob)


class IBool(schema.interfaces.IBool, IAttributeField):
    """Boolean Attribute Field.
    """

class IText(schema.interfaces.IText, IAttributeField):
    """Field containing an unicode string.
    """

class ITextLine(schema.interfaces.ITextLine, IAttributeField):
    """Field containing an unicode string without newlines.
    """

class IInt(schema.interfaces.IInt, IAttributeField):
    """Field containing an Integer Value.
    """

class IDatetime(schema.interfaces.IDatetime, IAttributeField):
    """Field containing a Datetime.
    """

class IDate(schema.interfaces.IDate, IAttributeField):
    """Field containing a Date.
    """

class ITimedelta(schema.interfaces.ITimedelta, IAttributeField):
    """Field containing a timedelta.
    """

class ITime(schema.interfaces.ITime, IAttributeField):
    """Field containing a time.
    """
    
class IURI(schema.interfaces.IURI, IAttributeField):
    """Field containing an absolute URI.
    """

class IRFC822MailAddress(email.interfaces.IRFC822MailAddress, IAttributeField):
    """A valid RFC822 email address field.
    """

class IObject(schema.interfaces.IObject, IAttributeField):
    """Field containing an Object value.
    """
