#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.component.interfaces import IObjectEvent
from zope.interface import Attribute, Interface
from zope.schema.vocabulary import SimpleTerm
from zope.app.container.constraints import containers, contains
from zope.app.container.interfaces import IContained, IContainer
from z3c.schema.email import RFC822MailAddress

from zw.status.interfaces import IStatusObject
from zw.status.state import State


class IHasParticipants(IAnnotatable):
    """Marker interface promising that an implementing object
    can have ''IParticipants'' annotations.
    """

class IParticipants(Interface):
    """The survey's participants.
    """

    def authenticateParticipant( uid ):
        """Check if uid is a valid participant and returns a PrincipalInfo.
        """


class IParticipantsManagement(Interface):
    """Helpers to manage the participants.
    """

    numberOfFinished = schema.Int(
        title = u"Number of participants who finshed the survey.",
        required = True ) 

    numberOfFailures = schema.Int(
        title = u"Number of participants whose invitation failed.",
        required = True ) 

##############################################################################

class IParticipantContainer(IContainer):
    """Containing a surveys participants.
    """
    contains('zw.vogon.participant.interfaces.IParticipant')
    
class IParticipant(IContained):
    """All information about a single participant.
    """
    containers('zw.vogon.participant.interfaces.IParticipantContainer')

    uid = schema.ASCIILine(
        title = _( u'label-participant.IParticipant.uid',
                   u"Participant's unique ID" ),
        description = _( u'help-participant.IParticipant.uid',
                         u"" ),
        required = True )

    email_address = RFC822MailAddress(
        title = _( u'label-participant.IParticipant.email_address',
                   u"Participant's email address" ),
        description = _( u'help-participant.IParticipant.email_address',
                         u"" ),
        required = True )
            
    anonymous = schema.Bool(
        title = _( u'label-participant.IParticipant.anonymous',
                   u"Anonymous participant" ),
        description = _( u'help-participant.IParticipant.anonymous',
                         u"If true the connection between participant and "
                         u"response will be removed after answering." ),
        default = False,
        required = True )

    remark = schema.Text(
        title = _( u'label-participant.IParticipant.remark',
                   u"Remark" ),
        description = _( u'help.participant.IParticipant.remark',
                         u"Infos e.g. about a failure." ),
        required = False )


    def getSurvey():
        """Try to look up the parental survey.
        """


class IStoreStatus(IStatusObject):
    """Annotation participant's status.
    """

class IParticipantStates(Interface):
    """
    """

    blank = State(
        title = _( u'label-participant.IParticipantStates.blank',
                   u"Blank" ),
        description = _( u'help-participant.IParticipantStates.blank',
                         u"This state should indicate that the the email address is not checked." ),
        default = True
        )

    ready = State(
        title = _( u'label-participant.IParticipantStates.ready',
                   u"Ready" ),
        description = _( u'help-participant.IParticipantStates.ready',
                         u"This indicates a participant ready to invite." )
        )

    active = State(
        title = _( u'label-participant.IParticipantStates.active',
                   u"Active" ),
        description = _( u'help-participant.IParticipantStates.active',
                         u"This indicates an active (invited) participant who is (still) able to fill in the survey." )
        )

    active_started_fillin = State(
        title = _( u'label-participant.IParticipantStates.active_stated_fillin',
                   u"Started filling in" ),
        description = _( u'help-participant.IParticipantStates.active_stated_fillin',
                         u"This indicates an active participant who has already begun filling in the survey." ),
        )

    failure = State(
        title = _( u'label-participant.IParticipantStates.failure',
                   u"Failure" ),
        description = _( u'help-participant.IParticipantStates.failure',
                         u"This indicates a problem with the email address or sending the invitation email. This participant is not active." ),
        )

    finished = State(
        title = _( u'label-participant.IParticipantStates.finished',
                   u"Finished" ),
        description = _( u'help-participant.IParticipantStates.finished',
                         u"An inactive participant who has already finished the survey." )
        )

    inactive = State(
        title = _( u'label-participant.IParticipantStates.inactive',
                   u"Inactive" ),
        description = _( u'help-participant.IParticipantStates.inactive',
                         u"An inactive participant who has never finished the form." )
        )

    inactive_started_fillin = State(
        title = _( u'label-participant.IParticipantStates.inactive_stated_fillin',
                   u"Inactive but started filling in" ),
        description = _( u'help-participant.IParticipantStates.inactive_stated_fillin',
                         u"An inactive participant who has already begun filling in the survey." ),
        )




class IParticipantStatus(Interface):
    """Possible participant's states.
    """

    errors = schema.Dict(
        title = _( u'label-participant.IParticipantStatus.errors',
                   u"Errors" ),
        description = _( u'help-participant.IParticipantStatus.errors',
                         u"Storing a list of errors abaout this participant. "
                         u"Usually concerns the sending of emails." ),
        required = False,
        key_type = schema.Datetime( 
            title = _(
                u'label-participant.IParticipantStatus.errors.timestamp',
                u"Timestamp" ),
            ),
        
        value_type = schema.Text(
            title = _(
                u'label-participant.IParticipantStatus.errors.entry',
                u"Entry" )
            )
        )


class IParticipantEvent(IObjectEvent):
    """State change of an participant
    """

    survey = schema.Object(
        title = u"Survey",
        schema = IHasParticipants )


class IParticipantActiveEvent(IParticipantEvent):
    """A participant has become active.
    """

class IParticipantInactiveEvent(IParticipantEvent):
    """A participant has become inactive.
    """

class IParticipantFinishedEvent(IParticipantEvent):
    """A participant has finished the survey.
    """

class IParticipantResetEvent(IParticipantEvent):
    """A participant has been reset.
    """


class IParticipantPrincipal(Interface):
    """
    """

    survey = Attribute( u"""The survey holding this participant.""") 
    participant = Attribute( u"""The according participang.""" )

class IParticipantPrincipalInfo(Interface):
    """
    """

    survey = Attribute( u"""The survey holding this participant.""" )
    participant = Attribute( u"""The according participang.""" )


##############################################################################
# Authentication & credential plugins

class IParticipantAuthentication(Interface):
    """This authentication utililty is able to lookup a participant
    of a registered survey.
    """


class IParticipantCredentials(Interface):
    """
    """

