#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import base64

from persistent import Persistent
from zope.cachedescriptors.property import Lazy
from zope.component import adapts, adapter, getUtility
from zope.event import notify
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.sendmail.interfaces import IMailDelivery
from zope.traversing.api import getParent
from zope.app.container.contained import Contained
from zope.app.container.interfaces import IObjectAddedEvent, IObjectRemovedEvent
from zope.app.intid.interfaces import IIntIds

from zw.status.interfaces import InvalidTransactionError
from zw.status.state import States
from zw.status.status import Status
from zw.status.transaction import transaction

from zw.vogon.portal.settings import IPortalSettings
from zw.vogon.survey.interfaces import ISurvey, IInvitationText, IReminderText
from zw.vogon.utils.b64 import urlsafe_b64decode
from zw.vogon.utils.mapping import MappingFieldProperty
from zw.vogon.participant.events import ParticipantActiveEvent, \
    ParticipantFinishedEvent, ParticipantResetEvent
from zw.vogon.participant.interfaces import IParticipant, \
    IParticipantStatus, IStoreStatus, IParticipantStates, IParticipants

class Participant(Contained, Persistent):
    implements(IParticipant)

    def __init__(self, **kwargs):
        super(Participant, self).__init__()
        for key, value in kwargs.items():
            if key in IParticipant:
                setattr(self, key, value)

    uid = FieldProperty(IParticipant['uid'])
    email_address = FieldProperty(IParticipant['email_address'])
    anonymous = FieldProperty(IParticipant['anonymous'])
    remark = FieldProperty(IParticipant['remark'])
    #status = FieldProperty(IParticipant['status'])

    def __repr__(self):
        return '<%s: %r>' % ( self.__class__.__name__, 
                              self.email_address )

    def getFolder(self):
        return getParent(self)

    @Lazy
    def survey(self):
        return getParent(self.getFolder())


@adapter(IParticipant, IObjectAddedEvent)
def prepareParticipant(participant, event):
    """Starts the workflow to assign a uid after adding.
    """
    status = IParticipantStatus(participant)
    try:
        status.startTransaction(status.prepare)
    except InvalidTransactionError:
        pass

@adapter(IParticipant, IObjectRemovedEvent)
def removeParticipant(participant, event):
    """Remove all responses of this participant
    """
    notify(ParticipantResetEvent(participant, participant.survey))


class ParticipantStatus(Status):
    implements(IParticipantStatus)
    adapts(IStoreStatus)

    KEY = 'zw.vogon.participant.status'

    states = States(IParticipantStates)
    errors = MappingFieldProperty(IParticipantStatus['errors'])

    @transaction(
        _( u'label-participant.ParticipantStatus.prepare',
           u"Prepare" ),
        description = _( u'help-participant.ParticipantStatus.prepare',
                         u"" ),
        states = ( ( IParticipantStates['blank'], IParticipantStates['ready'] ), ),
        )
    def prepare(self):
        # Create uid for participant
        participant = self.context
        if not participant.uid:
            folder = participant.getFolder()
            participant.uid = folder.generateId()
        

    @transaction(
        _( u'label-participant.ParticipantStatus.invite',
           u"Invite" ),
        description = _( u'help-participant.ParticipantStatus.invite',
                         u"" ),
        states = ( ( IParticipantStates['ready'], IParticipantStates['active'] ), )
        )
    def invite(self):
        mailer = getUtility(IMailDelivery, name=u'zw.vogon')
        intids = getUtility(IIntIds)
        #catalog = getUtility(ICatalog)
        portal_url = getUtility(IPortalSettings).base_url

        participant = self.context
        survey = participant.survey

        # 1st activate permissions for participant
        uid =  '%i_%s' % ( intids.getId(survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.assignRoleToPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )
        # 2st notify submodules (blank responses for this participant)
        notify( ParticipantActiveEvent(participant, survey) )

        # 3rd send invitation email
        mapping = {
            'url': '%s/participate.html?uid=%s' % (
                portal_url, base64.urlsafe_b64encode(uid) ) }
        msg = IInvitationText(survey).composeMessage( mapping )
        msg['To'] = participant.email_address
        msg['X-Vogon-Invite'] = base64.urlsafe_b64encode(uid)
        mailer.send( msg['From'], ( participant.email_address, ), 
                     msg.as_string() )
        

    @transaction(
        _( u'label-participant.ParticipantStatus.reportFailure',
           u"Report failure" ),
        description = _( u'help-participant.ParticipantStatus.reportFailure',
                         u"" ),
        states = ( ( IParticipantStates['active'], IParticipantStates['failure'] ), ),
        )
    def reportFailure(self):
        # Deactive this participant.

        intids = getUtility(IIntIds, context=self.context)
        participant = self.context
        survey = participant.survey

        uid =  '%i_%s' % ( intids.getId(participant.survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.unsetRoleForPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )

        # Remove responses
        notify( ParticipantResetEvent(participant, survey) )
        
        

    @transaction(
        _( u'label-participant.ParticipantStatus.startFillin',
           u"Start filling in the form" ),
        description = _( u'help-participant.ParticipantStatus.startFillin',
                         u"" ),
        states = ( ( IParticipantStates['active'], IParticipantStates['active_started_fillin'] ), )
        )
    def startFillin(self):
        """Doesn't do anything. It's only to indicate, that someone
        has begun to fill in the form.
        """
        pass

    @transaction(
        _( u'label-participant.ParticipantStatus.remind',
           u"Remind" ),
        description = _( u'help-participant.ParticipantStatus.remind',
                         u"" ),
        states = ( ( IParticipantStates['active'], IParticipantStates['active'] ),
                   ( IParticipantStates['active_started_fillin'], IParticipantStates['active_started_fillin'] ),)
        )
    def remind(self):
        mailer = getUtility(IMailDelivery, name=u'zw.vogon')
        intids = getUtility(IIntIds)
        portal_url = getUtility(IPortalSettings).base_url

        participant = self.context
        survey = participant.survey

        uid =  '%i_%s' % ( intids.getId(survey), participant.uid )
        mapping = {
            'url': '%s/participate.html?uid=%s' % (
                portal_url, base64.urlsafe_b64encode(uid) ) }
        msg = IReminderText(survey).composeMessage( mapping )
        msg['To'] = participant.email_address
        msg['X-Vogon-Reminder'] = base64.urlsafe_b64encode(uid)
        mailer.send( msg['From'], ( participant.email_address, ), 
                     msg.as_string() )


    @transaction(
        _( u'label-participant.ParticipantStatus.finish',
           u"Finish" ),
        description = _( u'help-participant.ParticipantStatus.finish',
                         u"" ),
        states = ( ( IParticipantStates['active_started_fillin'], IParticipantStates['finished'] ), )
        )
    def finish(self):
        intids = getUtility(IIntIds)
        participant = self.context
        survey = participant.survey

        uid =  '%i_%s' % ( intids.getId(participant.survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.unsetRoleForPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )

        notify(ParticipantFinishedEvent(participant, survey))

    @transaction(
        _( u'label-participant.ParticipantStatus.deactivate',
           u"Deactivate" ),
        description = _( u'help-participant.ParticipantStatus.deactivate',
                         u"" ),
        states = (
            ( IParticipantStates['active'], IParticipantStates['inactive'] ), 
            ( IParticipantStates['active_started_fillin'], IParticipantStates['inactive_started_fillin'] ), 
            )
        )
    def deactivate(self):
        intids = getUtility(IIntIds)
        participant = self.context
        survey = participant.survey

        uid =  '%i_%s' % ( intids.getId(participant.survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.unsetRoleForPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )

    @transaction(
        _( u'label-participant.ParticipantStatus.reactivate',
           u"Reactivate" ),
        description = _( u'help-participant.ParticipantStatus.reactivate',
                         u"" ),
        states = ( 
            ( IParticipantStates['inactive'], IParticipantStates['active'] ), 
            ( IParticipantStates['inactive_started_fillin'], IParticipantStates['active_started_fillin'] ), 
            )
        )
    def reactivate(self):
        intids = getUtility(IIntIds)
        participant = self.context
        survey = participant.survey

        uid =  '%i_%s' % ( intids.getId(survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.assignRoleToPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )


    @transaction(
        _( u'label-participant.ParticipantStatus.reset',
           u"Reset" ),
        description = _( u'help-participant.ParticipantStatus.reset',
                         u"" ),
        states = ( ( IParticipantStates['active'], IParticipantStates['ready'] ),
                   ( IParticipantStates['active_started_fillin'], IParticipantStates['ready'] ),
                   ( IParticipantStates['inactive_started_fillin'], IParticipantStates['ready'] ),
                   ( IParticipantStates['finished'], IParticipantStates['ready'] ),
                   ( IParticipantStates['inactive'], IParticipantStates['ready'] ), )
        )
    def reset(self):
        intids = getUtility(IIntIds)
        participant = self.context
        survey = participant.survey

        # remove permissions
        uid =  '%i_%s' % ( intids.getId(participant.survey), participant.uid )
        role_manager = IPrincipalRoleManager(survey)
        role_manager.unsetRoleForPrincipal( u'zw.vogon.Participant', 
                                            unicode(uid) )

        # delete all responses
        notify( ParticipantResetEvent(participant, survey) )
        
        


    #@transaction(
    #    _( u'label-participant.ParticipantStatus.',
    #       u"" ),
    #    description = _( u'help-participant.ParticipantStatus.',
    #                     u"" ),
    #    states = ( ( IParticipantStates[''], IParticipantStates[''] ), )
    #    )
    #def (self):
    #    pass


def participantInvitationFailed(portal, pid, reason=None):
    intid = getUtility(IIntIds, context=portal)
    s_id, p_id = urlsafe_b64decode(str(pid)).split('_')
    survey = intid.queryObject(int(s_id))
    participants = IParticipants(survey, None)
    if participants is None:
        return reason
    participant = participants.getParticipantByUid(p_id)
    if survey is None or participant is None:
        return reason

    p_status = IParticipantStatus(participant)
    try:
        p_status.startTransaction(p_status.reportFailure)

    except InvalidTransactionError:
        # This shouldn't happen
        return reason

    else:
        participant.remark = unicode(reason.as_string())
        
        
def participantRemindingFailed(portal, pid, reason=None):
    intid = getUtility(IIntIds, context=portal)
    s_id, p_id = urlsafe_b64decode(str(pid)).split('_')
    survey = intid.queryObject(int(s_id))
    participants = IParticipants(survey, None)
    if participants is None:
        return reason
    participant = participants.getParticipantByUid(p_id)
    if survey is None or participant is None:
        return reason
    
    participant.remark = u"Couldn't send reminder" #, reason.as_string()
    
