#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import createObject, getMultiAdapter, queryMultiAdapter, \
    getUtility
from zope.component.interfaces import IFactory
from zope.event import notify
from zope.interface import implements
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema import Choice
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.session.interfaces import ISession
from zope.traversing.api import getName, getParent
from zope.app.container.constraints import checkObject
from zope.app.container.interfaces import INameChooser
from zope.app.publisher.browser.menu import getMenu
from z3c.form import button, field, form, group, subform
from z3c.form.interfaces import INPUT_MODE, DISPLAY_MODE, IActionHandler

from zw.contentsform.cutcopypaste import CutCopyPasteMixin
from zw.vogon.question.control.interfaces import IControllable, \
    IControlledQuestion
from zw.vogon.question.interfaces import ISheetItem
from zw.vogon.question.browser.interfaces import IItemForm, \
    IAddItemForm, IManageItemForm, IManageItemButtons, \
    IItemGroupForm, IManageItemGroupForm, IAddableItems, \
    ISaveButton

_marker = object()


class ItemForm(form.Form):
    implements(IItemForm)

    __name__ = None
    mode = DISPLAY_MODE
    indices = []
    skip_indexing = False
    manage = False
    executedActions = []

    errors = None

    formErrorsMessage = _( u'There were some errors.')
    successMessage = ''
    noChangesMessage = ''


    def __init__(self, item, request, parentForm):
        super(ItemForm, self).__init__(item, request)
        self.__parent__ = self.parentForm = parentForm
        self.prefix = str( parentForm.prefix + getName(item) ) + '.'
        
    # Render some attributes
    @Lazy
    def label(self):
        return getattr( self.context, 'title', u'' )

    @Lazy
    def labelFor(self):
        if len(self.widgets):
            return self.widgets.values()[0].id
    def index(self):
        return u'.'.join( [unicode(i+1) for i in self.indices ] )+'.'


    def update(self):
        super(ItemForm, self).update()

        executedActions = []
        for action in self.parentForm.actions.executedActions + \
                getattr(self.parentForm, 'executedActions', []):
            adapter = queryMultiAdapter(
                (self, self.request, self.getContent(), action),
                interface=IActionHandler)
            if adapter:
                adapter()
                executedActions.append(action)
        self.executedActions = executedActions

        
    @button.handler(ISaveButton)
    def handleSaveButtons(self, action):
        data, errors = self.widgets.extract()
        if errors:
            self.status = self.formErrorsMessage
            self.errors = True
            return
        content = self.getContent()
        changed = form.applyChanges(self, content, data)
        if changed:
            notify(ObjectModifiedEvent(content))
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage
            

class ManageItemForm(ItemForm, form.EditForm):
    implements(IManageItemForm)

    manage = True

    def __init__(self, item, request, parentForm):
        super(ManageItemForm, self).__init__(item, request, parentForm)
        self.sessionKey = parentForm.sessionKey

    def update(self):
        if self.editing:
            self.mode = INPUT_MODE
        super(ManageItemForm, self).update()
        

    @Lazy
    def label(self):
        if self.editing:
            return _( u"Edit" )
        return super(ManageItemForm, self).label

    # Store attributes in the session key
    @Lazy
    def session(self):
        session = ISession(self.request)[self.sessionKey]
        if session.get('edit', _marker) is _marker:
            session['edit'] = set()
        if session.get('delete', _marker) is _marker:
            session['delete'] = None
        if session.get('add', _marker) is _marker:
            session['add'] = {}
        return session

    @Lazy
    def editing(self):
        return self.prefix in self.session.get('edit')
    @Lazy
    def delete(self):
        return self.prefix == self.session.get('delete')


    # Buttons and actions
    buttons = button.Buttons(IManageItemButtons)
    
    @button.handler( IManageItemButtons['edit'] )
    def handleEdit(self, action):
        self.session['edit'] = self.session['edit'].union([self.prefix])
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['save'] )
    def handleSave(self, action):
        self.handleApply(self, action)
        if self.editing:
            self.session['edit'].remove(self.prefix)
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['cancel_edit'] )
    def handleCancelEdit(self, action):
        if self.editing:
            self.session['edit'].remove(self.prefix)
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['up'] )
    def handleUp(self, action):
        parentForm = getParent(self)
        parentForm.itemsUp( getName(self.context) )
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['down'] )
    def handleDown(self, action):
        parentForm = getParent(self)
        parentForm.itemsDown( getName(self.context) )
        self.request.response.redirect(self.request.getURL())

        
    @button.handler( IManageItemButtons['delete_item'] )        
    def handleDelete(self, action):
        self.session['delete'] = self.prefix
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['confirm_delete'] )
    def handleConfirmDelete(self, action):
        parent = getParent(self.context)
        name = getName(self.context)
        del parent[name]
        self.session['delete'] = None
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageItemButtons['cancel_delete'] )
    def handleCancelDelete(self, action):
        self.session['delete'] = None
        self.request.response.redirect(self.request.getURL())


class AddItemForm(form.AddForm):
    implements(IAddItemForm)
    form.extends(form.AddForm)

    session = None
    factory_name = None

    def __init__(self, container, request, parentForm):
        super(AddItemForm, self).__init__(container, request)
        self.__parent__ = self.parentForm = parentForm
        self.prefix = str( parentForm.prefix + 'add-form' ) + '.'
        self.session = parentForm.session

        assert self.factory_name is not None, \
            u"Subclasses must provide a valid factory."

    def create(self, data):
        obj = createObject( self.factory_name )
        changes = form.applyChanges(self, obj, data)
        return obj

    def add(self, content):
        name = getattr(content, '__name__', None)
        chooser = INameChooser(self.context)

        # check precondition
        checkObject(self.context, name, content)

        name = chooser.chooseName(name or '', content)
        chooser.checkName(name, content)

        self.context[name] = content
        return self.context[name]

    def render(self):
        if self._finishedAdd:
            del self.parentForm.session['add'][self.parentForm.prefix]
            self.request.response.redirect(self.request.getURL())
            return ""
        return super(AddItemForm, self).render()


    @button.buttonAndHandler( _('Cancel'), name='cancel')
    def handleCancel(self, action):
        del self.parentForm.session['add'][self.parentForm.prefix]
        self.request.response.redirect(self.request.getURL())



class ItemGroupForm(ItemForm):
    implements(IItemGroupForm)

    subforms = []


    def getContents(self):
        return [item for item in self.context.values() \
                    if ISheetItem.providedBy(item) ]

    def update(self):
        super(ItemGroupForm, self).update()

        subforms = []
        c_skipped = 0
        for index, item in enumerate(self.getContents()):
            # Controlled
            try:
                controlled = IControlledQuestion(item)
            except:
                pass
            else:
                if controlled.isHidden(self.request.principal):
                    continue

            item_form = getMultiAdapter( (item, self.request, self), 
                                         IItemForm )
            item_form.indices = self.indices + [index-c_skipped]
            if item_form.skip_indexing:
                c_skipped += 1
            item_form.update()
            self.errors = item_form.errors or self.errors
            subforms.append(item_form)
            
        self.subforms = subforms



class SelectAddItemForm(form.Form):
    
    ignoreContext = True
    ignoreRequest = True

    def __init__(self, context, request, parentForm):
        self.context = context
        self.request = request
        self.parentForm = self.__parent__ = parentForm
        self.prefix = parentForm.prefix + 'add-selector.'

    label = u'Add'

    @Lazy
    def fields(self):
        return field.Fields(
            Choice(
                __name__ = 'add_type',
                title = _( u"Choose add type" ),
                required = False,
                vocabulary = SimpleVocabulary( tuple(
                        [ SimpleTerm( item['action'],
                                      title = _( item['action'],
                                                 item['title'] ) ) 
                          for item in getMenu('zope.app.container.add', 
                                              self.context, self.request) ] ) )
                )
            )

    def update(self):
        super(SelectAddItemForm, self).update()
        self.widgets['add_type'].prompt = True
        self.widgets['add_type'].update()


    @button.buttonAndHandler( _( u"Add" ), name='add' )
    def handleAdd(self, action):
        data, errors = self.extractData()
        factory = data.get('add_type')
        prefix = self.parentForm.prefix
        if factory is None:
            if prefix in self.parentForm.session['add']:
                del self.parentForm.session['add'][prefix]
        else:
            self.parentForm.session['add'][prefix] = factory
        self.request.response.redirect(self.request.getURL())
        
class ManageItemGroupForm(ItemGroupForm, ManageItemForm): 
    #, CutCopyPasteMixin):
    implements(IManageItemGroupForm)
    form.extends(ManageItemForm)
    
    addSelectionWidget = None

    def update(self):
        ManageItemForm.update(self)

        self.itemActions = [ item for name, item in self.actions.items()
                             if name in ManageItemForm.buttons.omit('save', 'cancel_edit').keys() ]
        self.groupActions = [ item for name, item in self.actions.items()
                              if name not in ManageItemForm.buttons.omit('save', 'cancel_edit').keys() ]

        subforms = []
        c_skipped = 0
        for index, item in enumerate(self.getContents()):
            item_form = getMultiAdapter( (item, self.request, self), 
                                         IManageItemForm )
            item_form.indices = self.indices + [index-c_skipped]
            if item_form.skip_indexing:
                c_skipped += 1
            item_form.update()
            subforms.append(item_form)


        if self.adding:
            factory_name = self.session['add'].get(self.prefix)
            item_add_form = getMultiAdapter( 
                (self.context, self.request, self),
                IAddItemForm, name=factory_name )
            item_add_form.update()
            subforms.append(item_add_form)

        self.subforms = subforms
            

        if not ( self.editing or self.adding ):
            addWidget = SelectAddItemForm(self.context, self.request, self)
            addWidget.update()
            self.addSelectionWidget = addWidget

                                      
    @Lazy
    def adding(self):
        return self.session['add'].get(self.prefix) is not None
            

    

    # Ordering facilities
    
    def itemsUp(self, *names):
        ind_keys = []
        for i, key in enumerate(self.context.keys()):
            i *= 2
            if key in names:
                i -= 3
            ind_keys.append( (i,key) )
        ind_keys.sort()
        self.context.updateOrder( tuple( map( lambda i: i[1], ind_keys ) ) )
                
    def itemsDown(self, *names):
        ind_keys = []
        for i, key in enumerate(self.context.keys()):
            i *= 2
            if key in names:
                i += 3
            ind_keys.append( (i,key) )
        ind_keys.sort()
        self.context.updateOrder( tuple( map( lambda i: i[1], ind_keys ) ) )


