#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import sys

from rwproperty import getproperty, setproperty
from persistent import Persistent
from zope.annotation.factory import factory as annotationFactory
from zope.cachedescriptors.property import Lazy
from zope.component import adapts, adapter
from zope.component.factory import Factory
from zope.event import notify
from zope.interface import alsoProvides, directlyProvides, implements, \
    implementedBy, classImplements, implementer
from zope.interface.declarations import Implements
from zope.lifecycleevent import ObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema import getFieldNames
from zope.schema.fieldproperty import FieldProperty
from zope.security.interfaces import IPrincipal
from zope.traversing.api import getParent
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import Contained, uncontained
from zope.app.container.interfaces import IObjectRemovedEvent

from zw.vogon.participant.interfaces import IParticipantPrincipal
from zw.vogon.question.interfaces import IResponsable, IResponse, \
    IResponseFolder, IResponseContainer, IResponseContained, \
    IResponseFactory, IStoreResponses


@implementer(IResponse)
@adapter(IResponsable, IPrincipal)
def getResponse(question, principal):
    if not IParticipantPrincipal.providedBy(principal):
        return ResponseDummy( question )
    
    responses = IResponseFolder(question)
    return responses.getResponseFor(principal)


_marker = object()


class ResponseDummy(object):
    implements(IResponse)

    def __init__(self, question):
        self.__question__ = question
        directlyProvides(self, question.response_schema)
        
        for name in getFieldNames(question.response_schema):
            setattr(self, name, question.response_schema[name].default)
            

class ResponseContained(Contained, Persistent):
    implements(IResponse, IResponseContained)
    
    __question__ = FieldProperty(IResponse['__question__'])
    participant = FieldProperty(IResponseContained['participant'])

    def __init__(self, question, participant_id):
        self.__question__ = question
        self.participant = participant_id
        directlyProvides(self, question.response_schema)
        

class ResponseFolder(BTreeContainer):
    implements(IResponseFolder, IResponseContainer)
    adapts(IStoreResponses)

    def getResponseFor(self, principal):
        question = getParent(self)

        if not IParticipantPrincipal.providedBy(principal):
            return ResponseDummy( question )

        try:
            # Return response for principal
            response = self[principal.id]

        except KeyError:
            # If we don't already have such a repsonse, create one.
            factory = IResponseFactory( question )
            response = factory(principal.id)
            notify(ObjectCreatedEvent(response))

            self[principal.id] = response
                    
        else:
            if not question.response_schema.providedBy(response):
                # XXX: Is this safe?
                directlyProvides(response, question.response_schema)
            
        return response

    def createBlankResponseFor(self, participant, force=False):
        question = getParent(self)
        id = participant.uid

        if force and id in self.keys():
            del self[id]

        if id not in self.keys():
            factory = IResponseFactory( question )
            response = factory(id)
            notify(ObjectCreatedEvent(response))

            self[id] = response
            
            return response
        
       


responseFolderFactory = annotationFactory( ResponseFolder, 
                                           'zw.vogon.question.responses')

@adapter(IStoreResponses, IObjectRemovedEvent)
def removeResponses(question, event):
    folder = IResponseFolder(question)
    for name, item in folder.items():
        uncontained(item, folder, name)


class ResponseFactory(Factory):
    implements(IResponseFactory)
    adapts(IResponsable)

    title = _( u'Response factory' )
    description = _( u'This factory instantiates new responses.' )
    _callable = ResponseContained

    def __init__(self, question):
        self.__question__ = question
        self._interfaces = (question.response_schema, 
                            IResponse, IResponseContained)
    
    def __call__(self, principal_id, *args, **kw):
        return self._callable(self.__question__, principal_id, *args, **kw)

    __question__ = FieldProperty(IResponseFactory['__question__'])

