#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from xml.sax.saxutils import quoteattr

from zope import i18n
from zope.cachedescriptors.property import Lazy
from zope.component import getAdapter, queryAdapter, getUtility, \
    ComponentLookupError
from zope.dublincore.interfaces import IZopeDublinCore
from zope.sendmail.interfaces import IMailDelivery
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.app.catalog.interfaces import ICatalog
from zc.table import column
from z3c.form import button, form

from zw.contentsform.adding import AddingFormMixin
from zw.contentsform.column import FieldColumn, Columns
from zw.contentsform.contentsform import ContentsForm
from zw.contentsform.cutcopypaste import CutCopyPasteMixin

from zw.status.interfaces import InvalidTransactionError

from zw.vogon.question.results.mail import MailResults

from zw.vogon.participant.interfaces import IParticipants, IParticipantStatus, \
    IParticipantStates
from zw.vogon.template.interfaces import IClone
from zw.vogon.survey.interfaces import ISurvey, ISurveyFolder, ISurveyArchive, \
    ISurveyStatus, ISurveyStates, ISurveySchedule


class TemplateColumn(column.Column):

    title = _( u"Used template" )
    name = u'template'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        ob = item.getContent()
        clone = queryAdapter(ob, IClone)
        
        if clone is None:
            return u''
        if clone.template is None:
            return u''

        #Check permission
        
        try:
            return u"<a href=%s>%s</a>" % ( 
                quoteattr( absoluteURL( clone.template, formatter.request ) ),
                clone.template.title )
        except TypeError:
            return u"%s (removed)" % clone.template.title 
            
        

class ResponseColumn(column.Column):

    title = u"Participants<br />fin. / tot (%)<br />&lt;Failures&gt;"
    name = u'response'

    def __init__(self):
        pass
    
    def renderCell(self, item, formatter):
        ob = item.getContent()
        participants = IParticipants(ob, None)
        if participants is None:
            return u""

        try:
            rel_finished = u"%.1f%%" % ( 100.*participants.numberOfFinished/(len(participants)) )
        except ZeroDivisionError:
            rel_finished = u" / "

        if participants.numberOfFailures > 0:
            return u"<a href=%s>%s</a>" % (
                quoteattr( absoluteURL( ob, formatter.request )+'/participants.html#failure' ),
                u"""<div style="white-space:nowrap; text-align:center;">%i / %i (%s)<br /><span style="color:red; font-weight: bold;">&lt;%i&gt;</span></div>""" % \
                    ( participants.numberOfFinished, len(participants), 
                      rel_finished, participants.numberOfFailures ) ) 

        return u"""<div style="white-space:nowrap; text-align:center;"><a href=%s>%s</a></div>""" % (
            quoteattr( absoluteURL( ob, formatter.request )+'/participants.html' ),
            u"%i / %i (%s)" % \
                ( participants.numberOfFinished, len(participants), 
                  rel_finished ) ) 


class StatusColumn(column.Column):

    title = u"Status"
    name = u'status'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        ob = item.getContent()
        obStatus = ISurveyStatus(ob, None)
        if obStatus is None:
            return u""
        return i18n.translate(
            obStatus.state.title, context = formatter.request )



class ResultsColumn(column.Column):

    title = _( u"Results" )
    name = u'results'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        ob = item.getContent()
        obStatus = ISurveyStatus(ob, None)
        if obStatus is not None:
            if obStatus.state in (ISurveyStates['finalised'], ISurveyStates['archived'],):
                try:
                    return u"""<a href=%s>PDF</a>, <a href=%s>Excel</a>""" % (
                        quoteattr( absoluteURL( ob, formatter.request )+'/results.pdf' ),
                        quoteattr( absoluteURL( ob, formatter.request )+'/results.xls' ) )
                
                except TypeError:
                    pass

        return u""
        
    

class ManageSurveysFolderForm(ContentsForm):
    
    cssClasses = {'table': 'listing' }

    def getContents(self):
        return [ item for item in self.context.values() \
                     if ISurveyFolder.providedBy(item) and \
                     not ISurveyArchive.providedBy(item) ]
    
    def _getColumns(self):
        return Columns(
            FieldColumn( ISurveyFolder['title'], action = 'contents.html' ),
            ResultsColumn(),
            )


                          
class ManageSurveysForm(AddingFormMixin, CutCopyPasteMixin, ContentsForm):
    form.extends(AddingFormMixin, CutCopyPasteMixin)
    
    @property
    def label(self):
        if self.context.title == u"Surveys":
            return _( u"Surveys overview" )
        else:
            return _( u"${title} - Surveys overview",
                      mapping = {'title': self.context.title } )

    cssClasses = {'table': 'listing' }

    # Since the are not implemented yet
    canCut = False
    canCopy = False
    canPaste = False

    subfolder_view = None

    def update(self):
        super(ManageSurveysForm, self).update()

        #QnD
        subfolder_view = ManageSurveysFolderForm(self.context, self.request)
        if subfolder_view.getContents():
            subfolder_view.update()
            self.subfolder_view = subfolder_view
        

    def getContents(self):
        ### Add surveyfolder ?
        return [ item for item in self.context.values() \
                     if ISurvey.providedBy(item) ]
    

    def _getColumns(self):
        return Columns(
            FieldColumn( ISurvey['title'], action = 'overview.html' ),
            ResponseColumn(),
            FieldColumn( ISurveyStatus['start'], readonly=True, addable=False ),
            FieldColumn( ISurveyStatus['stop'], readonly=True, addable=False ),
            #FieldColumn( IZopeDublinCore['created'], readonly=True, addable=False ),
            #FieldColumn( IZopeDublinCore['modified'], readonly=True, addable=False ),
            StatusColumn(),
            ResultsColumn(),
            TemplateColumn(),
            )

    @button.buttonAndHandler( _( u'Start' ), name='start',
                              condition=lambda form: form.getContents() )
    def handleStartSurveys(self, action):
        started = []
        invalid_status = []
        # collect selected surveys
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = ISurveyStatus(ob)
            try:
                status.startTransaction(status.start_survey)
            except InvalidTransactionError: 
                invalid_status.append(ob)
            else:
                started.append(ob)


    @button.buttonAndHandler( _( u'Remind' ), name='remind',
                              condition=lambda form: form.getContents() )
    def handleRemindSurveys(self, action):
        reminded = []
        invalid_status = [] 
        # collect selected surveys
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = ISurveyStatus(ob)
            try:
                status.startTransaction(status.remind)
            except InvalidTransactionError:
                invalid_status.append(ob)
            else:
                reminded.append(ob)


    @button.buttonAndHandler( _( u'Stop' ), name='stop',
                              condition=lambda form: form.getContents() )
    def handleStopSurveys(self, action):
        stopped = []
        invalid_status = []
        # collect selected surveys
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = ISurveyStatus(ob)
            try:
                status.startTransaction(status.stop_survey)
            except InvalidTransactionError:
                invalid_status.append(ob)
            else:
                stopped.append(ob)

    @button.buttonAndHandler( _( u'Restart' ), name='restart',
                              condition=lambda form: form.getContents() )
    def handleRestartSurveys(self, action):
        restarted = []
        invalid_status = [] 
        # collect selected surveys
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = ISurveyStatus(ob)
            try:
                status.startTransaction(status.restart)
            except InvalidTransactionError:
                invalid_status.append(ob)
            else:
                restarted.append(ob)

    @button.buttonAndHandler( _( u'Reset' ), name='reset',
                              condition=lambda form: form.getContents() )
    def handleResetSurveys(self, action): 
        reset = []
        invalid_status = [] 
        # collect selected surveys
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = ISurveyStatus(ob)
            try:
                status.startTransaction(status.reset)
            except InvalidTransactionError:
                invalid_status.append(ob)
            else:
                reset.append(ob)

    @button.buttonAndHandler( _( u'Send results' ), name='sendresults' )
    def handleSendResults(self, action):
        mailer = getUtility(IMailDelivery, name=u'zw.vogon')
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            mr = MailResults(ob)
            msg = mr.composeMessage({})
            if msg['To'] is None:
                print u"No address: %s" % ob.__name__
            else:
                mailer.send( msg['From'], ( msg['To'], ), 
                             msg.as_string() )

                
    #@button.buttonAndHandler( _( u'Archive' ), name='archive',
    #                          condition=lambda form: form.getContents() )
    #def handleArchiveSurveys(self, action):
    #    print "Archive the selected: niy"


