#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import getUtility
from zope.event import notify
from zope.interface import alsoProvides, implements, Interface
from zope.schema import getFieldNames
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.app.intid.interfaces import IIntIds
from zope.app.catalog.interfaces import ICatalog

from z3c.form import button
from z3c.formui import form
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.question.browser.interfaces import ISaveButton
from zw.vogon.question.control.interfaces import IControlledQuestion
from zw.vogon.question.sheet.browser.sheet_form import SheetForm

from zw.vogon.survey.events import SurveyFinishedEvent
from zw.vogon.survey.interfaces import ISurveyAttributes
from zw.vogon.survey.browser.interfaces import IParticipationForm, \
    IBackButton, IForwardButton, IStepButton, IHiddenButton



class IParticipationButtons(Interface):

    submitfix = button.ImageButton(
        title = u'submitfix',
        image = u'zw.contentsform/transparent-1x1.gif' )
    alsoProvides(submitfix, IHiddenButton)

    previous = button.Button( 
        title = _( u'Previous', ),
        condition = lambda form: not form.isFirstStep() )
    alsoProvides(previous, (ISaveButton, IBackButton))
    
    next = button.Button(
        title = _( u'Next', ),
        condition = lambda form: not form.isLastStep() )
    alsoProvides(next, (ISaveButton, IForwardButton))

    finish = button.Button(
        title = _( u'Finish', ),
        condition = lambda form: form.isLastStep() )
    alsoProvides(finish, (ISaveButton, IForwardButton))

class ParticipationButtonActions(button.ButtonActions):

    @property
    def hiddenActions(self):
        return [ action for action in self.values()
                 if IHiddenButton.providedBy(action.field) ]

    @property
    def backActions(self):
        return [ action for action in self.values()
                 if IBackButton.providedBy(action.field) ]

    @property
    def forwardActions(self):
        return [ action for action in self.values()
                 if IForwardButton.providedBy(action.field) ]

    @property
    def stepActions(self):
        return [ action for action in self.values()
                 if IStepButton.providedBy(action.field) ]



class CSSExtraPropertiesViewlet(object):

    extra_css = u''

    def update(self):
        # extract survey from request
        try:
            survey = self.request.principal.survey
        except AttributeError:
            self.request.response.redirect(self.request.getURL(1))
            return

        attributes = ISurveyAttributes(survey, None)
        if attributes is None:
            return
        self.extra_css = getattr(attributes, 'css', u'')

    def render(self):
        return u"""<style type="text/css">
  <!--
  #content {
      margin-left: 2em;
  }

  #site-actions {
      display: none;
  }
  
  #breadcrumbs {
      display: none;
  }
 
  %s
  -->
</style>""" % self.extra_css

class ParticipationForm(form.Form):
    implements(IParticipationForm)

    prefix = 'participation-form.'

    sheet = None
    sheets = ()
    sheet_pos = 0
    num_sheets = None
    
    survey = None
    redirect = False

    @property
    def label(self):
        return getattr( self.getContent(), 'title', u'' )

    @property
    def attributes(self):
        attributes = ISurveyAttributes(self.getContent(), None)
        if attributes is None:
            return []

        items = []
        schema = attributes.__schema__
        for name in getFieldNames(schema):
            value = getattr(attributes, name, None)
            if schema[name].public and value:
                items.append(
                    { 'title': schema[name].title,
                      'value': value } )
        return items

    @Lazy
    def buttons(self):
        return button.Buttons(IParticipationButtons)

    def update(self):
        # 1st get the survey
        survey = self.extractSurveyFromRequest()
        if survey is None:
            self.request.response.redirect(
                absoluteURL(self.context, self.request)+'/invalid_link.html' )
            self.redirect = True
            return

        # 2nd get the sheet position from the request
        self.num_sheets = len(survey)
        if self.num_sheets < 1:
            self.request.response.redirect(
                absoluteURL(self.context, self.request)+'/no_sheets.html' )
            self.redirect = True
            return
        try:
            sheet_pos = int(self.request.get('sheet_pos', 0))
            if sheet_pos >= self.num_sheets:
                raise ValueError
        except ValueError:
            self.sheet_pos = 0
        else:
            self.sheet_pos = sheet_pos
            

        # 3rd update my actions
        self.updateActions()


        # 4th adapt the sheet view
        sheets = []
        for index, (name, item) in enumerate( survey.items() ):
            sheet = SheetForm( item, self.request, self )
            sheet.indices = [ index ]
            if index == self.sheet_pos:
                sheet.update()
                self.sheet = sheet
            sheets.append( (name, sheet) )
        self.sheets = sheets

        self.actions.execute()


    def updateActions(self):
        self.actions = ParticipationButtonActions(self, self.request, self.context)
        self.actions.update()


    def getContent(self):
        # extract survey from request
        try:
            survey = self.request.principal.survey
        except AttributeError:
            self.request.response.redirect(self.request.getURL(1))
            return None
        else:
            return survey


    def extractSurveyFromRequest(self):
        try:
            survey = self.request.principal.survey
        except AttributeError:
            return None
        else:
            return survey
        


    def isFirstStep(self):
        return self.sheet_pos == 0


    def isLastStep(self):
        return self.sheet_pos == self.num_sheets-1

    @property
    def sheetsInfo(self):
        info = []
        for pos, (name, sheet) in enumerate(self.sheets):
            info.append( {
                    'name': name,
                    'number': str(pos+1),
                    'active': self.sheet == sheet,
                    'class': (self.sheet == sheet) and 'active' or 'inactive' } )
        return info

    @button.handler(IParticipationButtons['submitfix'])
    def handleSubmitFix(self, action):
        # Maybe we have to be subtler
        self.handleNext(self, action)

    @button.handler(IParticipationButtons['previous'])
    def handlePrevious(self, action):
        if self.sheet.errors:
            return
        
        # Skip IControllable-hidden sheets
        index = 0
        sheets = self.sheets[:self.sheet_pos]
        sheets.reverse()
        for index, (name, sheet) in enumerate(sheets):
            controlled = IControlledQuestion(sheet.context, None)
            if controlled is None:
                break
            if not controlled.isHidden(self.request.principal):
                break
            
        url = self.request.getURL(1) + \
            '/participate.html?uid=%s&sheet_pos=%i' % \
            ( self.request.get('uid', ''), self.sheet_pos-1-index )
        self.request.response.redirect(url)

    @button.handler(IParticipationButtons['next'])
    def handleNext(self, action):
        if self.sheet.errors:
            return

        # Skip IControllable-hidden sheets
        index = 0
        for index, (name, sheet) in enumerate( self.sheets[self.sheet_pos+1:] + [(None, None),] ):
            if sheet is None:
                self.handleFinish(self, action)
                return

            controlled = IControlledQuestion(sheet.context, None)
            if controlled is None:
                break
            
            if not controlled.isHidden(self.request.principal):
                break
                
        url = self.request.getURL(1) + \
            '/participate.html?uid=%s&sheet_pos=%i' % \
            ( self.request.get('uid', ''), self.sheet_pos+1+index )
        self.request.response.redirect(url)


    @button.handler(IParticipationButtons['finish'])
    def handleFinish(self, action):
        if self.sheet.errors:
            return

        # deactivate participant
        notify(SurveyFinishedEvent(
                self.getContent(),
                self.request.principal )
               )


        intids = getUtility(IIntIds)
        url = absoluteURL( self.context, self.request) + \
            '/finished.html?sid=%i' % intids.queryId( self.getContent() )
        self.request.response.redirect( url )


class FinishedSurvey(BrowserPagelet):

    @property
    def survey_title(self):
        intid = getUtility(IIntIds)
        try:
            survey = intid.queryObject( int(self.request.get('sid', None) ) )
        except TypeError:
            return None
        else:
            return survey.title
        
        
class InvalidSurvey(BrowserPagelet):

    @property
    def activeSurveys(self):
        return True
    #catalog = getUtility(ICatalog, name=u'catalog')
    #    index = catalog['workflow_state']
    #    return len(index.apply( (RUNNING, RUNNING) )) > 0
