#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.component.interfaces import IObjectEvent
from zope.deprecation import deprecated
from zope.interface import Interface
from zope.app.authentication.interfaces import IPrincipal
from zope.app.container.constraints import containers, contains
from zope.app.container.interfaces import IContained, IOrderedContainer

from zw.status.state import State
from zw.status.interfaces import IStatus, IStatusObject, ITransaction

class ISurvey(IContained):
    """Stores a survey.
    """
    containers('zw.vogon.survey.interfaces.ISurveyFolder')

    title = schema.TextLine(
        title = _( u'label-survey.ISurvey.title',
                   u"Title" ),
        description = _( u'help-survey.ISurvey.title',
                         u"Name of the survey" ),
        required = True )

class ISheetContainer(IOrderedContainer):
    """Stores question sheets.
    """
    contains('zw.vogon.question.sheet.interfaces.ISheet')


##############################################################################
# Survey folders

class ISurveyContainer(IOrderedContainer):
    """Store surveys
    """
    contains( 'zw.vogon.survey.interfaces.ISurveyFolder', 
              'zw.vogon.survey.interfaces.ISurvey' )


class ISurveyFolder(IContained):
    """Store surveys with possibility to create surveys from templates.
    """
    containers( 'zw.vogon.portal.interfaces.IPortalContainer',
                'zw.vogon.survey.interfaces.ISurveyContainer' )

    title = schema.TextLine(
        title = _( u'label-survey.ISurveyFolder.title',
                   u"Title" ),
        description = _( u'help-survey.ISurveyFolder.title',
                         u"Name of the folder" ),
        default = _( u"Surveys" ),
        required = True )


class ISurveyArchive(Interface):
    """A survey folder to archive old surveys.
    """


##############################################################################
# Survey management


class ISurveyStates(Interface):
    """Possible survey states
    """

    blank = State(
        title = _( u'label-survey.ISurveyStates.blank',
                   u"Blank" ),
        description = _( u'help-survey.ISurveyStates.blank',
                         u"This state should indicate that e.g. participants are still missing." ),
        default = True,
        )

    ready = State(
        title = _( u'label-survey.ISurveyStates.ready',
                   u"Ready" ),
        description = _( u'help-survey.ISurveyStates.ready',
                         u"" ),
        )

    running = State(
        title = _( u'label-survey.ISurveyStates.running',
                   u"Running" ),
        description = _( u'help-survey.ISurveyStates.running',
                         u"" ),
        )

    finalised = State(
        title = _( u'label-survey.ISurveyStates.finalised',
                   u"Finalised" ),
        description = _( u'help-survey.ISurveyStates.finalised',
                         u"" ),
        )

    archived = State(
        title = _( u'label-survey.ISurveyStates.archived',
                   u"Archived" ),
        description = _( u'help-survey.ISurveyStates.archived',
                         u"" ),
        )



class ISurveyStatus(IStatus):
    """Store the survey's state.
    """

    start = schema.Datetime(
        title = _( u'label-survey.ISurveyStatus.start',
                   u"Start" ),
        description = _( u'help-survey.ISurveyStatus.start',
                         u"" ),
        required = False )
        
    stop = schema.Datetime(
        title = _( u'label-survey.ISurveyStatus.stop',
                   u"Finish" ),
        description = _( u'help-survey.ISurveyStatus.stop',
                         u"" ),
        required = False )


    start_survey = schema.Object(
        title = u"Start survey",
        schema = ITransaction )


    stop_survey = schema.Object(
        title = u"Stop survey",
        schema = ITransaction )

    remind = schema.Object(
        title = u"Send reminder emails",
        schema = ITransaction )

    restart = schema.Object(
        title = u"Restart survey",
        schema = ITransaction )

    reset = schema.Object(
        title = u"Reset survey",
        schema = ITransaction )
    


class IStoreStatus(IStatusObject):
    """Annotate status and schedule.
    """


class ISurveyFinishedEvent(IObjectEvent):
    """
    """

    survey = schema.Object(
        title = u"Survey",
        schema = ISurvey )

    request = schema.Object(
        title = u"Principal",
        schema = IPrincipal )


class ISurveySchedule(Interface):
    """Store the survey's start and target date in the annotations.
    """

    auto = schema.Bool(
        title = _( u'label-survey.ISurveySchedule.auto',
                   u"Automatic schedule" ),
        description = _( u'help-survey-ISurveySchedule.auto',
                         u"" ),
        default = False )

    start = schema.Datetime(
        title = _( u'label-survey.ISurveySchedule.start',
                   u"Start" ),
        description = _( u'help-survey.ISurveySchedule.start',
                         u"" ),
        required = False )
        
    stop = schema.Datetime(
        title = _( u'label-survey.ISurveySchedule.stop',
                   u"Finish" ),
        description = _( u'help-survey.ISurveySchedule.stop',
                         u"" ),
        required = False )


class IStoreSchedule(IAnnotatable):
    """Annotate status and schedule.
    """



    


##############################################################################
# Mail texts 

class IStoreMailTexts(IAnnotatable):
    """Marker interface that promises an implementing object
    can have ''IMailText'' annotations.
    """

class IMailText(Interface):
    """Base class to invitation and reminder emails.

    Subclasses should overwrite the default values.
    """

    def composeMessage( mapping, use_defaults=True ):
        """Compose the subject by replacing the tags in mapping, 
        e.g. 
        mapping = { 'title':   u'Res stoica',
                    'persons': ( u'Zenon', u'Epiktet', ), }
        will replace tags <TITLE> and <PERSONS> by
        "Res stoica" and "Zenon and Epiktet".
        """

    subject = schema.TextLine(
        title = _( u"Subject" ) )

    body = schema.Text(
        title = _( u"Body" ) )


class IInvitationText(IMailText):
    """Adapter to store the invitation text.
    """

    subject = schema.TextLine(
        title = _( u'label-survey.IInvitationText.subject',
                   u"Invitation subject" ),
        description = _( u'help-survey.IInvitationText.subject',
                         "The 'subject' line of the invitation email. You can use the same tags as in the body below." ),
        required = False,
        default = _( u"Evaluation <TITLE>" )
        )

    body = schema.Text(
        title = _( u'label-survey.IInvitationText.body',
                   u"Invitation email text" ),
        description = _( u'help-survey.IInvitationText.body',
                         u"""This is the message text that will be sent
to the invited participant. The tags <URL>, <TITLE>, <PERSONS>, and
<ENDDATE> will be replaced by corresponding values of the survey.""" ),
        required = False,
        default = _( u"""Dear participant,

we are asking you to fill in the survey of <TITLE> by <PERSONS>
until <ENDDATE>. Please go to

  <URL>

Thanks a lot.
  Your evaluation team.""" )
        )


class IReminderText(IMailText):
    """Adapter to store the reminder text.
    """

    subject = schema.TextLine(
        title = _( u'label-survey.IReminderText.subject',
                   "Reminder subject" ),
        description = _( u'help-survey.IReminderText.subject',
                         u"You can use the same tags as in the body below." ),
        required = False,
        default = _( u"Evaluation <TITLE> (Reminder)" )
        )

    body = schema.Text(
        title = _( u'label-survey.IReminderText.body',
                   u"Reminder email text" ),
        description = _( u'help-survey.IReminderText.body',
                         u"""This is the message text that will be sent
to participant as reminder. The tags <URL>, <TITLE>, <PERSONS>, and
<ENDDATE> will be replaced by corresponding values of the survey.""" ),
        required = False,
        default = _( u"""Dear participant,

we are asking you to fill in the survey of <TITLE> by <PERSONS>
until <ENDDATE>. Please go to

  <URL>

Thanks a lot.
  Your evaluation team.""" )
        )


##############################################################################
# Attributes
# BBB

class IStoreSurveyAttributes(IAnnotatable):
    """Marker interface that promises an implementing object
    can have ''ISurveyAttributes'' annotations.
    """

class IBaseSurveyAttributesSchema(Interface):
    """Base schema
    """

class ISurveyAttributes(Interface):
    """Store the surveys attributes.
    """

    __schema__ = schema.InterfaceField(
        title = _( u'label-survey.ISurveyAttributes.__schema__',
                   u"Interface schema of attributes" ),
        description = _( u'help-survey.ISurveyAttributes.__schema__',
                         u"" ),
        required = True,
        default = IBaseSurveyAttributesSchema )


