#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from datetime import datetime

from zope.component import adapter, adapts
from zope.event import notify
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.app.container.ordered import OrderedContainer
from zope.app.container.interfaces import IObjectAddedEvent

from zw.status.interfaces import InvalidTransactionError
from zw.status.status import Status
from zw.status.state import States
from zw.status.transaction import transaction

from zw.vogon.utils.mapping import AnnotationsMapper, MappingFieldProperty

from zw.vogon.survey.interfaces import ISurvey, ISheetContainer, \
    ISurveyStates, ISurveyStatus, ISurveySchedule, IStoreSchedule, \
    IStoreStatus



class Survey(OrderedContainer):
    """The title is the only non-varying attribute. All others
    can be added by corresponding plugins.
    """
    implements(ISurvey, ISheetContainer)

    title = FieldProperty( ISurvey['title'] )


class SurveySchedule(AnnotationsMapper):
    implements(ISurveySchedule)
    adapts(IStoreSchedule)

    KEY = 'zw.vogon.survey.schedule'

    auto = MappingFieldProperty(ISurveySchedule['auto'])
    start = MappingFieldProperty(ISurveySchedule['start'])
    stop = MappingFieldProperty(ISurveySchedule['stop'])


@adapter(ISurvey, IObjectAddedEvent)
def prepareSurvey(survey, event):
    status = ISurveyStatus(survey)
    try:
        status.startTransaction(status.prepare)
    except InvalidTransactionError:
        pass
        

class SurveyStatus(Status):
    implements(ISurveyStatus)
    adapts(IStoreStatus)

    KEY = 'zw.vogon.survey.status'

    start = MappingFieldProperty(ISurveyStatus['start'])
    stop = MappingFieldProperty(ISurveyStatus['stop'])

    states = States(ISurveyStates)

    @transaction(
        _( u'label-survey.SurveyStatus.prepare',
           u"Prepare survey" ),
        description = _( u'help-survey.SurveyStatus.prepare',
                         u"Prepare the survey." ),
        states = ( (ISurveyStates['blank'], ISurveyStates['ready'] ), )
        )
    def prepare(self):
        """Perpare the survey.
        """
        pass

    @transaction(
        _( u'label-survey.SurveyStatus.start',
           u"Start survey" ), 
        description = _( u'help-survey.SurveyStatus.start',
                         u"Starts the survey and invites all participants" ),
        states = ( (ISurveyStates['ready'], ISurveyStates['running'] ), )
        )        
    def start_survey(self):
        """Start the survey.
        Invites all participants.
        """
        self.start = datetime.now()

    @transaction(
        _( u'label-survey.SurveyStatus.stop',
           u"Stop survey" ), 
        description = _( u'help-survey.SurveyStatus.stop',
                         u"Stopp the survey." ),
        states = ( (ISurveyStates['running'], ISurveyStates['finalised']), ),
        )        
    def stop_survey(self):
        """Stopps the survey.
        """
        self.stop = datetime.now()

    @transaction(
        _( u'label-survey.SurveyStatus.remind',
           u"Remind participants" ), 
        description = _( u'help-survey.SurveyStatus.remind',
                         u"Sends a reminder to all participants" ),
        states = ( ( ISurveyStates['running'], ISurveyStates['running'] ), ),
        )        
    def remind(self):
        """Reminds the participants.
        """
        pass

    @transaction(
        _( u'label-survey.SurveyStatus.restart',
           u"Restart survey" ), 
        description = _( u'help-survey.SurveyStatus.restart',
                         u"Restarts a stopped survey without reinviting participants." ),
        states = ( ( ISurveyStates['finalised'], ISurveyStates['running'] ), ),
        )        
    def restart(self):
        """Restarts a stopped survey.
        """
        self.stop = None

    @transaction(
        _( u'label-survey.SurveyStatus.reset',
           u"Reset survey" ), 
        description = _( u'help-survey.SurveyStatus.reset',
                         u"Resets the survey, i.e. delete all answers and reset participants." ),
        states = ( (ISurveyStates['running'], ISurveyStates['ready']),
                   (ISurveyStates['finalised'], ISurveyStates['ready']), ),
        )        
    def reset(self):
        """Resets the survey, i.e. delete all answers and reset participants.
        """
        self.start = None
        self.stop = None

    #@transaction(
    #    _( u'label-survey.SurveyStatus.delHistory',
    #       u"Delete logs" ), 
    #    description = _( u'help-survey.SurveyStatus.delHistory',
    #                     u"Delete the log." ),
    #    states = ( ISurveyStates['ready'], ),
    #    )        
    #def delHistory(self):
    #    """Remove the log.
    #    """


