#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
""" Mapping (field) property classes for annotation adapters.
$Id$
"""
__docformat__ = 'reStructuredText'

from persistent.dict import PersistentDict
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.interface import classImplements
from zope.schema import getFields
from zope.schema.fieldproperty import FieldProperty

_marker = object()


class AnnotationsMapperClass(type):
    """A metaclass to create an annotation adapter in KEY with
    fields defined by a schema.
    """
    
    def __new__(cls, name, KEY, schema, class_=None, adapts=(), *args, **kw):
        if type(class_) is tuple:
            bases = class_ + (AnnotationsMapper, )
        elif class_ is not None:
            bases = (class_, AnnotationsMapper, ) 
        else:
            bases = (AnnotationsMapper, )

        tp = type.__new__(cls, name, bases, {})

        return tp
        

    def __init__(cls, name, KEY, schema, class_=None, adapts=(), *args, **kw):

        setattr(cls, 'KEY', KEY)

        for field_id in getFields(schema):
            if not hasattr(cls, field_id):
                setattr(cls, field_id, MappingFieldProperty(schema[field_id]))

        classImplements(cls, schema)
        if adapts:
            _adapter = adapter(adapts)
            _adapter(cls)

class AnnotationsMapper(object):

    KEY = None

    def __init__(self, context):
        self.__parent__ = self.context = context
        assert self.KEY is not None, \
            "Subclasses have to specify a KEY."
        annotations = IAnnotations(context)
        mapping = annotations.get(self.KEY)
        if mapping is None:
            mapping = annotations[self.KEY] = PersistentDict()
        self.mapping = mapping



class MappingProperty(object):

    def __init__(self, name):
        self.name = name

    def __get__(self, inst, class_=None):
        return inst.mapping[self.name]

    def __set__(self, inst, value):
        inst.mapping[self.name] = value


class MappingFieldProperty(object):

    def __init__(self, field, name=None):
        if name is None:
            name = field.__name__

            self.__field = field
            self.__name = name

    def __get__(self, inst, klass):
        if inst is None:
            return self

        value = inst.mapping.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)

        return value


    def __set__(self, inst, value):
        field = self.__field.bind(inst)
        field.validate(value)
        if field.readonly and inst.mapping.has_key(self.__name):
            raise ValueError(self.__name, 'field is readonly')
        inst.mapping[self.__name] = value

    def __getattr__(self, name):
        return getattr(self.__field, name)

